#include "mgos.h"

#ifdef MGOS_HAVE_DHT
#include "main.h"
#include "mgos_dht.h"
#include "mgos_config.h"
#include "mqtt.h"

#define MAX_DHT 8

static struct mgos_dht *s_dht[MAX_DHT];
static int s_num_dht = 0;

static void dht_cb(void *ud) {
  int dht_idx = (int)ud;
  float t, h;

  if (!s_dht[dht_idx]) {
    LOG(LL_ERROR, ("No DHT handle to work with!"));
    return;
  }
  t = mgos_dht_get_temp(s_dht[dht_idx]);
  h = mgos_dht_get_humidity(s_dht[dht_idx]);

  mqtt_publish_stat("dht", "{idx:%d, temp:%.2f, humidity:%.1f}", dht_idx, t, h);
}

static bool dht_create(int pin, enum dht_type type) {
  struct mgos_dht *dht;
  if (s_num_dht == MAX_DHT) {
    LOG(LL_ERROR, ("No more sensor slots available (%d added)", MAX_DHT));
    return false;
  }

  dht=mgos_dht_create(pin, type);
  if (!dht) {
    LOG(LL_ERROR, ("Could not create DHT sensor on pin %d", pin));
    return false;
  }
  s_dht[s_num_dht] = dht;
  mgos_set_timer(5000, true, dht_cb, (void *)s_num_dht);
  s_num_dht++;

  return true;
}

void dht_init() {
  char *tok;

  memset(s_dht, 0, sizeof (struct mgos_dht *) * MAX_DHT);
  tok = strtok((char *)mgos_sys_config_get_app_dht_gpio(), ", ");
  while (tok) {
    int gpio;
    gpio = atoi(tok);
    tok=strtok(NULL, ", ");
    dht_create(gpio, AM2302);
    mgos_msleep(250);
  }
}

#else
void dht_init() {
  LOG(LL_INFO, ("DHT disabled, include library in mos.yml to enable"));
}
#endif
